@@ -577,8 +577,12 @@ panel_widget_cadd (GtkContainer *container,
 
 	p = g_object_get_data (G_OBJECT(widget),
 			       PANEL_APPLET_ASSOC_PANEL_KEY);
-	if(p)
-		run_up_forbidden(p,add_panel_to_forbidden);
+	if (p) {
+		panel_toplevel_attach_to_widget (p->toplevel,
+						 PANEL_WIDGET (container)->toplevel,
+						 widget);
+		run_up_forbidden (p, add_panel_to_forbidden);
+	}
 }
 
 static void
@@ -597,8 +601,10 @@ panel_widget_cremove (GtkContainer *container, GtkWidget *widget)
 	p = g_object_get_data (G_OBJECT (widget),
 				 PANEL_APPLET_ASSOC_PANEL_KEY);
 
-	if (p != NULL)
+	if (p != NULL) {
+		panel_toplevel_detach (p->toplevel);
 		run_up_forbidden (p, remove_panel_from_forbidden);
+	}
 
 	panel_widget_reset_focus (container, widget);
 
@@ -700,6 +706,7 @@ allocate_dirty_child(gpointer data)
 	AppletData *ad = data;
 	AppletData *nad;
 	PanelWidget *panel;
+	GtkWidget *widget;
 	GtkAllocation challoc;
 	GtkRequisition chreq;
 	
@@ -707,10 +714,22 @@ allocate_dirty_child(gpointer data)
 	g_return_val_if_fail(ad->applet != NULL,FALSE);
 	g_return_val_if_fail(PANEL_IS_WIDGET(ad->applet->parent),FALSE);
 
-	panel = PANEL_WIDGET(ad->applet->parent);
+	panel = PANEL_WIDGET (ad->applet->parent);
+	widget = GTK_WIDGET (panel);
 	
 	if(!ad->dirty)
 		return FALSE;
+
+	/* If a packed panel is not given all the space it wants
+	   and this is an expanding applet we must reallocate everything */
+	if (panel->packed &&
+	    ad->expand_major &&
+	    (widget->requisition.width > widget->allocation.width ||
+	     widget->requisition.height > widget->allocation.height)) {
+		gtk_widget_queue_resize (widget);
+		return FALSE;
+	}
+		
 	
 	gtk_widget_get_child_requisition(ad->applet,&chreq);
 
@@ -1330,7 +1349,7 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 		gtk_widget_size_request(ad->applet,&chreq);
 
 		if (panel->orient == GTK_ORIENTATION_HORIZONTAL) {
-			if (requisition->height < chreq.height)
+			if (requisition->height < chreq.height && !ad->size_constrained)
 				requisition->height = chreq.height;
 
 			if (panel->packed && ad->expand_major && ad->size_hints)
@@ -1339,7 +1358,7 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 			else if (panel->packed)
 				requisition->width += chreq.width;
 		} else {
-			if (requisition->width < chreq.width)
+			if (requisition->width < chreq.width && !ad->size_constrained)
 				requisition->width = chreq.width;
 
 			if (panel->packed && ad->expand_major && ad->size_hints)
@@ -1358,8 +1377,10 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 		}
 	} 
 	
-	requisition->width = CLAMP (requisition->width, 12, gdk_screen_width ());
-	requisition->height = CLAMP (requisition->height, 12, gdk_screen_height ());
+	if (requisition->width < 12)
+		requisition->width = 12;
+	if (requisition->height < 12)
+		requisition->height = 12;
 }
 
 static void
@@ -1393,6 +1414,22 @@ panel_widget_set_background_region (PanelWidget *panel)
 		widget->allocation.height);
 }
 
+static int
+panel_widget_count_expanded_applets (PanelWidget *panel)
+{
+	GList *li;
+	int count = 0;
+
+	for (li = panel->applet_list; li != NULL; li = li->next) {
+		AppletData *ad = li->data;
+
+		if (ad->expand_major)
+			count ++;
+	}
+
+	return count;
+}
+
 static void
 panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -1428,7 +1465,22 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	if(old_size<panel->size)
 		panel_widget_right_stick(panel,old_size);
 
-	if(panel->packed) {
+	if (panel->packed) {
+		int compression = 0;
+		int applets_to_compress = 0;
+
+		if (panel->orient == GTK_ORIENTATION_HORIZONTAL &&
+		    widget->requisition.width > allocation->width) {
+			compression = widget->requisition.width - allocation->width;
+		} else if (panel->orient == GTK_ORIENTATION_VERTICAL &&
+			   widget->requisition.height > allocation->height) {
+			compression = widget->requisition.height - allocation->height;
+		}
+
+		if (compression > 0) {
+			applets_to_compress = panel_widget_count_expanded_applets (panel);
+		}
+
 		i = 0;
 		for(list = panel->applet_list;
 		    list!=NULL;
@@ -1446,10 +1498,17 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				if (ad->expand_minor)
 					challoc.height = allocation->height;
 
-				if (ad->expand_major && ad->size_hints)
-					challoc.width = CLAMP (MAX (ad->size_hints [0], chreq.width),
+				if (ad->expand_major && ad->size_hints) {
+					int width = ad->size_hints [0];
+					if (applets_to_compress > 0) {
+						width -= (compression / applets_to_compress);
+						compression -= (compression / applets_to_compress);
+						applets_to_compress --;
+					}
+					challoc.width = CLAMP (MAX (width, chreq.width),
 							       chreq.width,
 							       allocation->width - i);
+				}
 
 				ad->cells = challoc.width;
 				challoc.x = ad->constrained;
@@ -1458,10 +1517,17 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				if (ad->expand_minor)
 					challoc.width = allocation->width;
 
-				if (ad->expand_major && ad->size_hints)
-					challoc.height = CLAMP (MAX (ad->size_hints [0], chreq.height),
+				if (ad->expand_major && ad->size_hints) {
+					int height = ad->size_hints [0];
+					if (applets_to_compress > 0) {
+						height -= (compression / applets_to_compress);
+						compression -= (compression / applets_to_compress);
+						applets_to_compress --;
+					}
+					challoc.height = CLAMP (MAX (height, chreq.height),
 								chreq.height,
 								allocation->height - i);
+				}
 
 				ad->cells = challoc.height;
 				challoc.x = allocation->width / 2 - challoc.width / 2;
@@ -1676,6 +1742,11 @@ panel_widget_destroy (GtkObject *obj)
 
 	panels = g_slist_remove (panels, panel);
 
+	if (panel->master_widget != NULL)
+		g_object_set_data (G_OBJECT (panel->master_widget),
+				   PANEL_APPLET_ASSOC_PANEL_KEY,
+				   NULL);
+
 	if (GTK_OBJECT_CLASS (panel_widget_parent_class)->destroy)
 		GTK_OBJECT_CLASS (panel_widget_parent_class)->destroy (obj);
 }
@@ -1836,7 +1907,8 @@ panel_widget_applet_drag_start (PanelWidget *panel,
 		gdk_flush ();
 
 		if (status != GDK_GRAB_SUCCESS) {
-			g_warning (G_STRLOC ": failed to grab pointer");
+			g_warning (G_STRLOC ": failed to grab pointer (errorcode: %d)",
+				   status);
 			panel_widget_applet_drag_end (panel);
 		}
 	}
@@ -2214,9 +2286,13 @@ panel_widget_applet_event(GtkWidget *widget, GdkEvent *event)
 
 			if ( ! commie_mode &&
 			    bevent->button == 2) {
+				guint32 time_ = bevent->time;
+				/* time on sent events seems to be bogus */
+				if (bevent->send_event)
+					time_ = GDK_CURRENT_TIME;
 				/* Start drag */
 				panel_widget_applet_drag_start (
-					panel, widget, PW_DRAG_OFF_CURSOR, bevent->time);
+					panel, widget, PW_DRAG_OFF_CURSOR, time_);
 				return TRUE;
 			}
 
@@ -2462,6 +2538,7 @@ panel_widget_add (PanelWidget *panel,
 		ad->drag_off = 0;
 		ad->dirty = FALSE;
 		ad->no_die = 0;
+		ad->size_constrained = FALSE;
 		ad->expand_major = FALSE;
 		ad->expand_minor = FALSE;
 		ad->locked = locked;
@@ -2768,6 +2845,27 @@ panel_widget_get_applet_orientation (PanelWidget *panel)
 	return panel_toplevel_get_orientation (panel->toplevel);
 }
 
+void
+panel_widget_set_applet_size_constrained (PanelWidget *panel,
+					  GtkWidget   *applet,
+					  gboolean     size_constrained)
+{
+	AppletData *ad;
+
+	ad = g_object_get_data (G_OBJECT (applet), PANEL_APPLET_DATA);
+	if (!ad)
+		return;
+
+	size_constrained = size_constrained != FALSE;
+
+	if (ad->size_constrained == size_constrained)
+		return;
+
+	ad->size_constrained = size_constrained;
+
+	gtk_widget_queue_resize (GTK_WIDGET (panel));
+}
+
 void
 panel_widget_set_applet_expandable (PanelWidget *panel,
 				    GtkWidget   *applet,
